# coding: utf-8

"""
    Rapidata.Dataset

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictBytes, StrictStr
from typing import Optional, Tuple, Union
from typing_extensions import Annotated
from rapidata.api_client.models.inquire_file_metadata_result import InquireFileMetadataResult
from rapidata.api_client.models.upload_file_result import UploadFileResult

from rapidata.api_client.api_client import ApiClient, RequestSerialized
from rapidata.api_client.api_response import ApiResponse
from rapidata.api_client.rest import RESTResponseType


class AssetApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def asset_file_name_metadata_get(
        self,
        file_name: Annotated[StrictStr, Field(description="The name of the file to retrieve metadata for.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> InquireFileMetadataResult:
        """Gets the metadata for an asset by file name.

        This endpoint retrieves metadata information for an asset stored in S3. The metadata includes details such as content type, size, creation date, and custom metadata fields.

        :param file_name: The name of the file to retrieve metadata for. (required)
        :type file_name: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._asset_file_name_metadata_get_serialize(
            file_name=file_name,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "InquireFileMetadataResult",
            '404': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def asset_file_name_metadata_get_with_http_info(
        self,
        file_name: Annotated[StrictStr, Field(description="The name of the file to retrieve metadata for.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[InquireFileMetadataResult]:
        """Gets the metadata for an asset by file name.

        This endpoint retrieves metadata information for an asset stored in S3. The metadata includes details such as content type, size, creation date, and custom metadata fields.

        :param file_name: The name of the file to retrieve metadata for. (required)
        :type file_name: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._asset_file_name_metadata_get_serialize(
            file_name=file_name,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "InquireFileMetadataResult",
            '404': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def asset_file_name_metadata_get_without_preload_content(
        self,
        file_name: Annotated[StrictStr, Field(description="The name of the file to retrieve metadata for.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Gets the metadata for an asset by file name.

        This endpoint retrieves metadata information for an asset stored in S3. The metadata includes details such as content type, size, creation date, and custom metadata fields.

        :param file_name: The name of the file to retrieve metadata for. (required)
        :type file_name: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._asset_file_name_metadata_get_serialize(
            file_name=file_name,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "InquireFileMetadataResult",
            '404': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _asset_file_name_metadata_get_serialize(
        self,
        file_name,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if file_name is not None:
            _path_params['fileName'] = file_name
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/plain', 
                    'application/json', 
                    'text/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'bearer', 
            'oauth2'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/asset/{fileName}/metadata',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def asset_file_post(
        self,
        file: Annotated[Optional[Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]]], Field(description="The file to upload to S3.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UploadFileResult:
        """Uploads a single asset to S3 and returns the asset details.

        This endpoint allows uploading a single file to S3 storage with the asset creator. The uploaded asset will be stored using the configured S3 settings and a unique filename will be generated.

        :param file: The file to upload to S3.
        :type file: bytearray
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._asset_file_post_serialize(
            file=file,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UploadFileResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def asset_file_post_with_http_info(
        self,
        file: Annotated[Optional[Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]]], Field(description="The file to upload to S3.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UploadFileResult]:
        """Uploads a single asset to S3 and returns the asset details.

        This endpoint allows uploading a single file to S3 storage with the asset creator. The uploaded asset will be stored using the configured S3 settings and a unique filename will be generated.

        :param file: The file to upload to S3.
        :type file: bytearray
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._asset_file_post_serialize(
            file=file,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UploadFileResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def asset_file_post_without_preload_content(
        self,
        file: Annotated[Optional[Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]]], Field(description="The file to upload to S3.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Uploads a single asset to S3 and returns the asset details.

        This endpoint allows uploading a single file to S3 storage with the asset creator. The uploaded asset will be stored using the configured S3 settings and a unique filename will be generated.

        :param file: The file to upload to S3.
        :type file: bytearray
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._asset_file_post_serialize(
            file=file,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UploadFileResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _asset_file_post_serialize(
        self,
        file,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        if file is not None:
            _files['file'] = file
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/plain', 
                    'application/json', 
                    'text/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'multipart/form-data'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'bearer', 
            'oauth2'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/asset/file',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def asset_url_post(
        self,
        url: Annotated[Optional[StrictStr], Field(description="The url of the file to upload to S3.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UploadFileResult:
        """Uploads a single asset to S3 and returns the asset details.

        This endpoint allows uploading a single file to S3 storage with the asset creator. The uploaded asset will be stored using the configured S3 settings and a unique filename will be generated.

        :param url: The url of the file to upload to S3.
        :type url: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._asset_url_post_serialize(
            url=url,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UploadFileResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def asset_url_post_with_http_info(
        self,
        url: Annotated[Optional[StrictStr], Field(description="The url of the file to upload to S3.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UploadFileResult]:
        """Uploads a single asset to S3 and returns the asset details.

        This endpoint allows uploading a single file to S3 storage with the asset creator. The uploaded asset will be stored using the configured S3 settings and a unique filename will be generated.

        :param url: The url of the file to upload to S3.
        :type url: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._asset_url_post_serialize(
            url=url,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UploadFileResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def asset_url_post_without_preload_content(
        self,
        url: Annotated[Optional[StrictStr], Field(description="The url of the file to upload to S3.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Uploads a single asset to S3 and returns the asset details.

        This endpoint allows uploading a single file to S3 storage with the asset creator. The uploaded asset will be stored using the configured S3 settings and a unique filename will be generated.

        :param url: The url of the file to upload to S3.
        :type url: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._asset_url_post_serialize(
            url=url,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UploadFileResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _asset_url_post_serialize(
        self,
        url,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if url is not None:
            
            _query_params.append(('url', url))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/plain', 
                    'application/json', 
                    'text/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'bearer', 
            'oauth2'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/asset/url',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





from rapidata.api_client.models.static_selection import (
    StaticSelection as StaticSelectionModel,
)
from rapidata.rapidata_client.selection._base_selection import RapidataSelection


class StaticSelection(RapidataSelection):
    """StaticSelection Class

    Given a list of RapidIds, theses specific rapids will be shown in order for every session.

    Args:
        rapid_ids (list[str]): List of rapid ids to show.
    """

    def __init__(self, rapid_ids: list[str]):
        self.rapid_ids = rapid_ids

    def __str__(self) -> str:
        return f"StaticSelection(rapid_ids={self.rapid_ids})"

    def __repr__(self) -> str:
        return self.__str__()



from rapidata.rapidata_client.settings import (
    AlertOnFastResponse,
    TranslationBehaviour,
    FreeTextMinimumCharacters,
    NoShuffle,
    PlayVideoUntilTheEnd,
    AllowNeitherBoth,
    SwapContextInstruction,
    MuteVideo,
)


class RapidataSettings:
    """
    Container class for all setting factory functions

    Settings can be added to an order to determine the behaviour of the task.

    Attributes:
        alert_on_fast_response (AlertOnFastResponse): Gives an alert as a pop up on the UI when the response time is less than the milliseconds.
        translation_behaviour (TranslationBehaviour): Defines what's the behaviour of the translation in the UI.
        free_text_minimum_characters (FreeTextMinimumCharacters): Only for free text tasks. Set the minimum number of characters a user has to type.
        no_shuffle (NoShuffle): Only for classification and compare tasks. If true, the order of the categories / images will not be shuffled and presented in the same order as specified.
        play_video_until_the_end (PlayVideoUntilTheEnd): Allows users to only answer once the video has finished playing.
        allow_neither_both (AllowNeitherBoth): Only for compare tasks. If true, the users will be able to select neither or both instead of exclusively one of the options.
        swap_context_instruction (SwapContextInstruction): Swap the place of the context and instruction.
        mute_video (MuteVideo): Mute the video.

    Example:
        ```python
        from rapidata import FreeTextMinimumCharacters
        settings=[FreeTextMinimumCharacters(10)]
        ```

        This can be used in a free text order to set the minimum number of characters required to submit the task.
    """

    alert_on_fast_response = AlertOnFastResponse
    translation_behaviour = TranslationBehaviour
    free_text_minimum_characters = FreeTextMinimumCharacters
    no_shuffle = NoShuffle
    play_video_until_the_end = PlayVideoUntilTheEnd
    allow_neither_both = AllowNeitherBoth
    swap_context_instruction = SwapContextInstruction
    mute_video = MuteVideo

    def __str__(self) -> str:
        return f"RapidataSettings(alert_on_fast_response={self.alert_on_fast_response}, translation_behaviour={self.translation_behaviour}, free_text_minimum_characters={self.free_text_minimum_characters}, no_shuffle={self.no_shuffle}, play_video_until_the_end={self.play_video_until_the_end}, allow_neither_both={self.allow_neither_both}, swap_context_instruction={self.swap_context_instruction}, mute_video={self.mute_video})"

    def __repr__(self) -> str:
        return self.__str__()



from rapidata.rapidata_client.datapoints._datapoint import Datapoint
from rapidata.service.openapi_service import OpenAPIService
from rapidata.api_client.models.create_datapoint_model import CreateDatapointModel
from rapidata.api_client.models.create_datapoint_model_asset import (
    CreateDatapointModelAsset,
)
from rapidata.api_client.models.create_datapoint_model_context_asset import (
    CreateDatapointModelContextAsset,
)
from rapidata.api_client.models.create_datapoint_result import CreateDatapointResult
from rapidata.rapidata_client.datapoints._asset_uploader import AssetUploader


class DatapointUploader:
    def __init__(self, openapi_service: OpenAPIService):
        self.openapi_service = openapi_service
        self.asset_uploader = AssetUploader(openapi_service)

    def upload_datapoint(
        self, datapoint: Datapoint, dataset_id: str, index: int
    ) -> CreateDatapointResult:

        uploaded_asset = (
            self._handle_media_datapoint(datapoint)
            if datapoint.data_type == "media"
            else self._handle_text_datapoint(datapoint)
        )
        return self.openapi_service.dataset_api.dataset_dataset_id_datapoint_post(
            dataset_id=dataset_id,
            create_datapoint_model=CreateDatapointModel(
                asset=uploaded_asset,
                context=datapoint.context,
                contextAsset=(
                    CreateDatapointModelContextAsset(
                        actual_instance=self.asset_uploader.get_uploaded_asset_input(
                            datapoint.media_context
                        )
                    )
                    if datapoint.media_context
                    else None
                ),
                transcription=datapoint.sentence,
                sortIndex=index,
                group=datapoint.group,
            ),
        )

    def _handle_text_datapoint(self, datapoint: Datapoint) -> CreateDatapointModelAsset:
        return CreateDatapointModelAsset(
            actual_instance=self.asset_uploader.get_uploaded_text_input(
                datapoint.asset
            ),
        )

    def _handle_media_datapoint(
        self, datapoint: Datapoint
    ) -> CreateDatapointModelAsset:
        return CreateDatapointModelAsset(
            actual_instance=self.asset_uploader.get_uploaded_asset_input(
                datapoint.asset
            ),
        )



from rapidata.rapidata_client.selection._base_selection import RapidataSelection
from rapidata.api_client.models.effort_capped_selection import (
    EffortCappedSelection as EffortCappedSelectionModel,
)
from rapidata.rapidata_client.selection.rapidata_retrieval_modes import (
    RapidataRetrievalMode,
)


class EffortSelection(RapidataSelection):
    """
    With this selection you can define the effort budget you have for a task.
    As an example, you have a task that takes 10 seconds to complete. The effort budget would be 10.

    Args:
        effort_budget (int): The effort budget for the task.
        retrieval_mode (RetrievalMode): The retrieval mode for the task.
        max_iterations (int | None): The maximum number of iterations for the task.
    """

    def __init__(
        self,
        effort_budget: int,
        retrieval_mode: RapidataRetrievalMode = RapidataRetrievalMode.Shuffled,
        max_iterations: int | None = None,
    ):
        self.effort_budget = effort_budget
        self.retrieval_mode = retrieval_mode
        self.max_iterations = max_iterations

    def _to_model(self):
        return EffortCappedSelectionModel(
            _t="EffortCappedSelection",
            effortBudget=self.effort_budget,
            retrievalMode=self.retrieval_mode.value,
            maxIterations=self.max_iterations,
        )

    def __str__(self) -> str:
        return f"EffortSelection(effort_budget={self.effort_budget}, retrieval_mode={self.retrieval_mode}, max_iterations={self.max_iterations})"

    def __repr__(self) -> str:
        return self.__str__()



from pydantic import BaseModel, Field


class OrderConfig(BaseModel):
    """
    Holds the configuration for the order process.

    Attributes:
        minOrderDatapointsForValidation (int): The minimum number of datapoints required so that an automatic validationset gets created if no recommended was found. Defaults to 50.
        autoValidationSetSize (int): The maximum size of the auto-generated validation set. Defaults to 20.
    """

    autoValidationSetCreation: bool = Field(default=True)
    minOrderDatapointsForValidation: int = Field(default=20)



import pandas as pd
import urllib.parse
import webbrowser
from colorama import Fore
from typing import Literal, Optional, Sequence

from rapidata.api_client.models.and_user_filter_model_filters_inner import (
    AndUserFilterModelFiltersInner,
)
from rapidata.api_client.models.create_benchmark_participant_model import (
    CreateBenchmarkParticipantModel,
)
from rapidata.api_client.models.create_leaderboard_model import CreateLeaderboardModel
from rapidata.api_client.models.file_asset_model import FileAssetModel
from rapidata.api_client.models.query_model import QueryModel
from rapidata.api_client.models.page_info import PageInfo
from rapidata.api_client.models.source_url_metadata_model import SourceUrlMetadataModel
from rapidata.api_client.models.submit_prompt_model import SubmitPromptModel
from rapidata.api_client.models.existing_asset_input import ExistingAssetInput
from rapidata.api_client.models.create_datapoint_model_context_asset import (
    CreateDatapointModelContextAsset,
)
from rapidata.rapidata_client.benchmark._detail_mapper import DetailMapper
from rapidata.rapidata_client.benchmark.leaderboard.rapidata_leaderboard import (
    RapidataLeaderboard,
)
from rapidata.rapidata_client.benchmark.participant._participant import (
    BenchmarkParticipant,
)
from rapidata.rapidata_client.datapoints._asset_uploader import AssetUploader
from rapidata.rapidata_client.filter import RapidataFilter
from rapidata.rapidata_client.config import logger, managed_print, tracer
from rapidata.rapidata_client.settings import RapidataSetting
from rapidata.service.openapi_service import OpenAPIService


class RapidataBenchmark:
    """
    An instance of a Rapidata benchmark.

    Used to interact with a specific benchmark in the Rapidata system, such as retrieving prompts and evaluating models.

    Args:
        name: The name that will be used to identify the benchmark on the overview.
        id: The id of the benchmark.
        openapi_service: The OpenAPI service to use to interact with the Rapidata API.
    """

    def __init__(self, name: str, id: str, openapi_service: OpenAPIService):
        self.name = name
        self.id = id
        self._openapi_service = openapi_service
        self.__prompts: list[str | None] = []
        self.__prompt_assets: list[str | None] = []
        self.__leaderboards: list[RapidataLeaderboard] = []
        self.__identifiers: list[str] = []
        self.__tags: list[list[str]] = []
        self.__benchmark_page: str = (
            f"https://app.{self._openapi_service.environment}/mri/benchmarks/{self.id}"
        )
        self._asset_uploader = AssetUploader(openapi_service)

    def __instantiate_prompts(self) -> None:
        with tracer.start_as_current_span("RapidataBenchmark.__instantiate_prompts"):
            current_page = 1
            total_pages = None

            while True:
                prompts_result = self._openapi_service.benchmark_api.benchmark_benchmark_id_prompts_get(
                    benchmark_id=self.id,
                    request=QueryModel(page=PageInfo(index=current_page, size=100)),
                )

                if prompts_result.total_pages is None:
                    raise ValueError(
                        "An error occurred while fetching prompts: total_pages is None"
                    )

                total_pages = prompts_result.total_pages

                for prompt in prompts_result.items:
                    self.__prompts.append(prompt.prompt)
                    self.__identifiers.append(prompt.identifier)
                    if prompt.prompt_asset is None:
                        self.__prompt_assets.append(None)
                    else:
                        assert isinstance(
                            prompt.prompt_asset.actual_instance, FileAssetModel
                        )
                        source_url = prompt.prompt_asset.actual_instance.metadata[
                            "sourceUrl"
                        ].actual_instance
                        assert isinstance(source_url, SourceUrlMetadataModel)
                        self.__prompt_assets.append(source_url.url)

                    self.__tags.append(prompt.tags)
                if current_page >= total_pages:
                    break

                current_page += 1

    @property
    def identifiers(self) -> list[str]:
        if not self.__identifiers:
            self.__instantiate_prompts()

        return self.__identifiers

    @property
    def prompts(self) -> list[str | None]:
        """
        Returns the prompts that are registered for the leaderboard.
        """
        if not self.__prompts:
            self.__instantiate_prompts()

        return self.__prompts

    @property
    def prompt_assets(self) -> list[str | None]:
        """
        Returns the prompt assets that are registered for the benchmark.
        """
        if not self.__prompt_assets:
            self.__instantiate_prompts()

        return self.__prompt_assets

    @property
    def tags(self) -> list[list[str]]:
        """
        Returns the tags that are registered for the benchmark.
        """
        if not self.__tags:
            self.__instantiate_prompts()

        return self.__tags

    @property
    def leaderboards(self) -> list[RapidataLeaderboard]:
        """
        Returns the leaderboards that are registered for the benchmark.
        """
        with tracer.start_as_current_span("RapidataBenchmark.leaderboards"):
            if not self.__leaderboards:
                current_page = 1
                total_pages = None

                while True:
                    leaderboards_result = self._openapi_service.benchmark_api.benchmark_benchmark_id_leaderboards_get(
                        benchmark_id=self.id,
                        request=QueryModel(
                            page=PageInfo(index=current_page, size=100),
                        ),
                    )

                    if leaderboards_result.total_pages is None:
                        raise ValueError(
                            "An error occurred while fetching leaderboards: total_pages is None"
                        )

                    total_pages = leaderboards_result.total_pages

                    self.__leaderboards.extend(
                        [
                            RapidataLeaderboard(
                                leaderboard.name,
                                leaderboard.instruction,
                                leaderboard.show_prompt,
                                leaderboard.show_prompt_asset,
                                leaderboard.is_inversed,
                                leaderboard.response_budget,
                                leaderboard.min_responses,
                                self.id,
                                leaderboard.id,
                                self._openapi_service,
                            )
                            for leaderboard in leaderboards_result.items
                        ]
                    )

                    if current_page >= total_pages:
                        break

                    current_page += 1

            return self.__leaderboards

    def add_prompt(
        self,
        identifier: str | None = None,
        prompt: str | None = None,
        prompt_asset: str | None = None,
        tags: Optional[list[str]] = None,
    ):
        """
        Adds a prompt to the benchmark.

        Args:
            identifier: The identifier of the prompt/asset/tags that will be used to match up the media. If not provided, it will use the prompt, asset or prompt + asset as the identifier.
            prompt: The prompt that will be used to evaluate the model.
            prompt_asset: The prompt asset that will be used to evaluate the model. Provided as a link to the asset.
            tags: The tags can be used to filter the leaderboard results. They will NOT be shown to the users.
        """
        with tracer.start_as_current_span("RapidataBenchmark.add_prompt"):
            if tags is None:
                tags = []

            if prompt is None and prompt_asset is None:
                raise ValueError("Prompt or prompt asset must be provided.")

            if identifier is None and prompt is None:
                raise ValueError("Identifier or prompt must be provided.")

            if identifier and not isinstance(identifier, str):
                raise ValueError("Identifier must be a string.")

            if prompt and not isinstance(prompt, str):
                raise ValueError("Prompt must be a string.")

            if prompt_asset and not isinstance(prompt_asset, str):
                raise ValueError(
                    "Asset must be a string. That is the link to the asset."
                )

            if identifier is None:
                assert prompt is not None
                if prompt in self.prompts:
                    raise ValueError(
                        "Prompts must be unique. Otherwise use identifiers."
                    )
                identifier = prompt

            if identifier in self.identifiers:
                raise ValueError("Identifier already exists in the benchmark.")

            if tags is not None and (
                not isinstance(tags, list)
                or not all(isinstance(tag, str) for tag in tags)
            ):
                raise ValueError("Tags must be a list of strings.")

            logger.info(
                "Adding identifier %s with prompt %s, prompt asset %s and tags %s to benchmark %s",
                identifier,
                prompt,
                prompt_asset,
                tags,
                self.id,
            )

            self.__identifiers.append(identifier)

            self.__tags.append(tags)
            self.__prompts.append(prompt)
            self.__prompt_assets.append(prompt_asset)

            self._openapi_service.benchmark_api.benchmark_benchmark_id_prompt_post(
                benchmark_id=self.id,
                submit_prompt_model=SubmitPromptModel(
                    identifier=identifier,
                    prompt=prompt,
                    promptAsset=(
                        CreateDatapointModelContextAsset(
                            actual_instance=ExistingAssetInput(
                                _t="ExistingAssetInput",
                                name=self._asset_uploader.upload_asset(prompt_asset),
                            )
                        )
                        if prompt_asset is not None
                        else None
                    ),
                    tags=tags,
                ),
            )

    def create_leaderboard(
        self,
        name: str,
        instruction: str,
        show_prompt: bool = False,
        show_prompt_asset: bool = False,
        inverse_ranking: bool = False,
        level_of_detail: Literal["low", "medium", "high", "very high"] = "low",
        min_responses_per_matchup: int = 3,
        validation_set_id: str | None = None,
        filters: Sequence[RapidataFilter] = [],
        settings: Sequence[RapidataSetting] = [],
    ) -> RapidataLeaderboard:
        """
        Creates a new leaderboard for the benchmark.

        Args:
            name: The name of the leaderboard. (not shown to the users)
            instruction: The instruction decides how the models will be evaluated.
            show_prompt: Whether to show the prompt to the users. (default: False)
            show_prompt_asset: Whether to show the prompt asset to the users. (only works if the prompt asset is a URL) (default: False)
            inverse_ranking: Whether to inverse the ranking of the leaderboard. (if the question is inversed, e.g. "Which video is worse?")
            level_of_detail: The level of detail of the leaderboard. This will effect how many comparisons are done per model evaluation. (default: "low")
            min_responses_per_matchup: The minimum number of responses required to be considered for the leaderboard. (default: 3)
            validation_set_id: The id of the validation set that should be attached to the leaderboard. (default: None)
            filters: The filters that should be applied to the leaderboard. Will determine who can solve answer in the leaderboard. (default: [])
            settings: The settings that should be applied to the leaderboard. Will determine the behavior of the tasks on the leaderboard. (default: [])
        """
        with tracer.start_as_current_span("create_leaderboard"):
            if not isinstance(min_responses_per_matchup, int):
                raise ValueError("Min responses per matchup must be an integer")

            if min_responses_per_matchup < 3:
                raise ValueError("Min responses per matchup must be at least 3")

            logger.info(
                "Creating leaderboard %s with instruction %s, show_prompt %s, show_prompt_asset %s, inverse_ranking %s, level_of_detail %s, min_responses_per_matchup %s, validation_set_id %s, filters %s, settings %s",
                name,
                instruction,
                show_prompt,
                show_prompt_asset,
                inverse_ranking,
                level_of_detail,
                min_responses_per_matchup,
                validation_set_id,
                filters,
                settings,
            )

            leaderboard_result = self._openapi_service.leaderboard_api.leaderboard_post(
                create_leaderboard_model=CreateLeaderboardModel(
                    benchmarkId=self.id,
                    name=name,
                    instruction=instruction,
                    showPrompt=show_prompt,
                    showPromptAsset=show_prompt_asset,
                    isInversed=inverse_ranking,
                    minResponses=min_responses_per_matchup,
                    responseBudget=DetailMapper.get_budget(level_of_detail),
                    validationSetId=validation_set_id,
                    filters=(
                        [
                            AndUserFilterModelFiltersInner(filter._to_model())
                            for filter in filters
                        ]
                        if filters
                        else None
                    ),
                    featureFlags=(
                        [setting._to_feature_flag() for setting in settings]
                        if settings
                        else None
                    ),
                )
            )

            assert (
                leaderboard_result.benchmark_id == self.id
            ), "The leaderboard was not created for the correct benchmark."

            logger.info("Leaderboard created with id %s", leaderboard_result.id)

            return RapidataLeaderboard(
                name,
                instruction,
                show_prompt,
                show_prompt_asset,
                inverse_ranking,
                leaderboard_result.response_budget,
                min_responses_per_matchup,
                self.id,
                leaderboard_result.id,
                self._openapi_service,
            )

    def evaluate_model(
        self,
        name: str,
        media: list[str],
        identifiers: list[str] | None = None,
        prompts: list[str] | None = None,
    ) -> None:
        """
        Evaluates a model on the benchmark across all leaderboards.

        prompts or identifiers must be provided to match the media.

        Args:
            name: The name of the model.
            media: The generated images/videos that will be used to evaluate the model.
            identifiers: The identifiers that correspond to the media. The order of the identifiers must match the order of the media.\n
                The identifiers that are used must be registered for the benchmark. To see the registered identifiers, use the identifiers property.
            prompts: The prompts that correspond to the media. The order of the prompts must match the order of the media.
        """
        with tracer.start_as_current_span("evaluate_model"):
            if not media:
                raise ValueError("Media must be a non-empty list of strings")

            if not identifiers and not prompts:
                raise ValueError("Identifiers or prompts must be provided.")

            if identifiers and prompts:
                raise ValueError(
                    "Identifiers and prompts cannot be provided at the same time. Use one or the other."
                )

            if not identifiers:
                assert prompts is not None
                identifiers = prompts

            if len(media) != len(identifiers):
                raise ValueError(
                    "Media and identifiers/prompts must have the same length"
                )

            if not all(identifier in self.identifiers for identifier in identifiers):
                raise ValueError(
                    "All identifiers/prompts must be in the registered identifiers/prompts list. To see the registered identifiers/prompts, use the identifiers/prompts property."
                )

            participant_result = self._openapi_service.benchmark_api.benchmark_benchmark_id_participants_post(
                benchmark_id=self.id,
                create_benchmark_participant_model=CreateBenchmarkParticipantModel(
                    name=name,
                ),
            )

            logger.info(f"Participant created: {participant_result.participant_id}")

            participant = BenchmarkParticipant(
                name, participant_result.participant_id, self._openapi_service
            )

            with tracer.start_as_current_span("upload_media_for_participant"):
                logger.info(
                    f"Uploading {len(media)} media assets to participant {participant.id}"
                )

                successful_uploads, failed_uploads = participant.upload_media(
                    media,
                    identifiers,
                )

                total_uploads = len(media)
                success_rate = (
                    (len(successful_uploads) / total_uploads * 100)
                    if total_uploads > 0
                    else 0
                )
                logger.info(
                    f"Upload complete: {len(successful_uploads)} successful, {len(failed_uploads)} failed ({success_rate:.1f}% success rate)"
                )

                if failed_uploads:
                    logger.error(f"Failed uploads for media: {failed_uploads}")
                    logger.warning(
                        "Some uploads failed. The model evaluation may be incomplete."
                    )

                if len(successful_uploads) == 0:
                    raise RuntimeError(
                        "No uploads were successful. The model evaluation will not be completed."
                    )

                self._openapi_service.participant_api.participants_participant_id_submit_post(
                    participant_id=participant_result.participant_id
                )

    def view(self) -> None:
        """
        Views the benchmark.
        """
        logger.info("Opening benchmark page in browser...")
        could_open_browser = webbrowser.open(self.__benchmark_page)
        if not could_open_browser:
            encoded_url = urllib.parse.quote(
                self.__benchmark_page, safe="%/:=&?~#+!$,;'@()*[]"
            )
            managed_print(
                Fore.RED
                + f"Please open this URL in your browser: '{encoded_url}'"
                + Fore.RESET
            )

    def get_overall_standings(self, tags: Optional[list[str]] = None) -> pd.DataFrame:
        """
        Returns an aggregated elo table of all leaderboards in the benchmark.
        """
        with tracer.start_as_current_span("get_overall_standings"):
            participants = self._openapi_service.benchmark_api.benchmark_benchmark_id_standings_get(
                benchmark_id=self.id,
                tags=tags,
            )

            standings = []
            for participant in participants.items:
                standings.append(
                    {
                        "name": participant.name,
                        "wins": participant.wins,
                        "total_matches": participant.total_matches,
                        "score": (
                            round(participant.score, 2)
                            if participant.score is not None
                            else None
                        ),
                    }
                )

            return pd.DataFrame(standings)

    def __str__(self) -> str:
        return f"RapidataBenchmark(name={self.name}, id={self.id})"

    def __repr__(self) -> str:
        return self.__str__()



from pydantic import BaseModel, Field


class UploadConfig(BaseModel):
    """
    Holds the configuration for the upload process.

    Attributes:
        maxWorkers (int): The maximum number of worker threads for processing media paths. Defaults to 10.
        maxRetries (int): The maximum number of retries for failed uploads. Defaults to 3.
    """

    maxWorkers: int = Field(default=10)
    maxRetries: int = Field(default=3)
    chunkSize: int = Field(default=50)
    cacheUploads: bool = Field(default=True)



from rapidata.api_client.models.simple_workflow_model import SimpleWorkflowModel
from rapidata.api_client.models.simple_workflow_model_blueprint import (
    SimpleWorkflowModelBlueprint,
)
from rapidata.api_client.models.transcription_rapid_blueprint import (
    TranscriptionRapidBlueprint,
)
from rapidata.rapidata_client.workflow._base_workflow import Workflow
from rapidata.api_client import TranscriptionPayload, TranscriptionWord
from rapidata.rapidata_client.datapoints._datapoint import Datapoint
from rapidata.api_client.models.rapid_modality import RapidModality


class SelectWordsWorkflow(Workflow):
    """
    A workflow for select words tasks.

    This class represents a select words workflow
    where datapoints have a sentence attached to them where words can be selected.

    Attributes:
        _instruction (str): The instruction for the select words task.

    Args:
        instruction (str): The instruction to be provided for the select words task.
    """

    modality = RapidModality.TRANSCRIPTION

    def __init__(self, instruction: str):
        super().__init__(type="SimpleWorkflowConfig")
        self._instruction = instruction

    def _get_instruction(self) -> str:
        return self._instruction

    def _to_model(self) -> SimpleWorkflowModel:
        blueprint = TranscriptionRapidBlueprint(
            _t="TranscriptionBlueprint", title=self._instruction
        )

        return SimpleWorkflowModel(
            _t="SimpleWorkflow",
            blueprint=SimpleWorkflowModelBlueprint(blueprint),
        )

    def _to_payload(self, datapoint: Datapoint) -> TranscriptionPayload:
        assert (
            datapoint.sentence is not None
        ), "SelectWordsWorkflow requires a sentence datapoint"

        return TranscriptionPayload(
            _t="TranscriptionPayload",
            title=self._instruction,
            transcription=[
                TranscriptionWord(word=word, wordIndex=i)
                for i, word in enumerate(datapoint.sentence.split())
            ],
        )

    def __str__(self) -> str:
        return f"SelectWordsWorkflow(instruction='{self._instruction}')"

    def __repr__(self) -> str:
        return f"SelectWordsWorkflow(instruction={self._instruction!r})"



import logging
from typing import Protocol, runtime_checkable
from opentelemetry._logs import set_logger_provider
from opentelemetry.exporter.otlp.proto.http._log_exporter import OTLPLogExporter
from opentelemetry.sdk._logs import LoggerProvider, LoggingHandler
from opentelemetry.sdk._logs.export import BatchLogRecordProcessor
from opentelemetry.sdk.resources import Resource
from rapidata import __version__
from .logging_config import LoggingConfig, register_config_handler


@runtime_checkable
class LoggerProtocol(Protocol):
    """Protocol that defines the logger interface for type checking."""

    def debug(self, msg: object, *args, **kwargs) -> None: ...
    def info(self, msg: object, *args, **kwargs) -> None: ...
    def warning(self, msg: object, *args, **kwargs) -> None: ...
    def warn(self, msg: object, *args, **kwargs) -> None: ...
    def error(self, msg: object, *args, **kwargs) -> None: ...
    def exception(self, msg: object, *args, exc_info=True, **kwargs) -> None: ...
    def critical(self, msg: object, *args, **kwargs) -> None: ...
    def fatal(self, msg: object, *args, **kwargs) -> None: ...
    def log(self, level: int, msg: object, *args, **kwargs) -> None: ...
    def isEnabledFor(self, level: int) -> bool: ...
    def getEffectiveLevel(self) -> int: ...
    def setLevel(self, level: int | str) -> None: ...
    def addHandler(self, handler: logging.Handler) -> None: ...
    def removeHandler(self, handler: logging.Handler) -> None: ...
    @property
    def handlers(self) -> list[logging.Handler]: ...
    @property
    def level(self) -> int: ...
    @property
    def name(self) -> str: ...


class RapidataLogger:
    """Logger implementation that updates when the configuration changes."""

    def __init__(self, name: str = "rapidata"):
        self._logger = logging.getLogger(name)
        self._otlp_initialized = False
        self._otlp_handler = None

        # Register this logger to receive configuration updates
        register_config_handler(self._handle_config_update)

    def _handle_config_update(self, config: LoggingConfig) -> None:
        """Handle configuration updates."""
        self._update_logger(config)

    def _update_logger(self, config: LoggingConfig) -> None:
        """Update the logger based on the new configuration."""
        # Initialize OTLP logging only once and only if not disabled
        if not self._otlp_initialized and config.enable_otlp:
            try:
                logger_provider = LoggerProvider(
                    resource=Resource.create(
                        {
                            "service.name": "Rapidata.Python.SDK",
                            "service.version": __version__,
                        }
                    ),
                )
                set_logger_provider(logger_provider)

                exporter = OTLPLogExporter(
                    endpoint="https://otlp-sdk.rapidata.ai/v1/logs",
                    timeout=30,
                )

                processor = BatchLogRecordProcessor(
                    exporter,
                    max_queue_size=2048,
                    export_timeout_millis=30000,
                    max_export_batch_size=512,
                )

                logger_provider.add_log_record_processor(processor)

                # OTLP handler - captures DEBUG and above
                self._otlp_handler = LoggingHandler(logger_provider=logger_provider)
                self._otlp_handler.setLevel(logging.DEBUG)  # OTLP gets everything

                self._otlp_initialized = True

            except Exception as e:
                self._logger.warning(f"Failed to initialize OTLP logging: {e}")
                import traceback

                traceback.print_exc()

        # Console handler with configurable level
        console_handler = logging.StreamHandler()
        console_level = getattr(logging, config.level.upper())
        console_handler.setLevel(console_level)
        console_formatter = logging.Formatter(config.format)
        console_handler.setFormatter(console_formatter)

        # Configure the logger
        self._logger.setLevel(logging.DEBUG)  # Logger must allow DEBUG for OTLP

        # Remove any existing handlers (except OTLP when appropriate)
        for handler in self._logger.handlers[:]:
            if handler != self._otlp_handler:
                self._logger.removeHandler(handler)
            elif handler == self._otlp_handler and not config.enable_otlp:
                self._logger.removeHandler(handler)

        # Add OTLP handler if initialized and not disabled
        if (
            self._otlp_handler
            and self._otlp_handler not in self._logger.handlers
            and config.enable_otlp
        ):
            self._logger.addHandler(self._otlp_handler)

        # Add console handler
        self._logger.addHandler(console_handler)

        # Add file handler if log_file is provided
        if config.log_file:
            file_handler = logging.FileHandler(config.log_file)
            file_handler.setLevel(console_level)  # Use same level as console
            file_formatter = logging.Formatter(config.format)
            file_handler.setFormatter(file_formatter)
            self._logger.addHandler(file_handler)

    def __getattr__(self, name: str) -> object:
        """Delegate attribute access to the underlying logger."""
        return getattr(self._logger, name)


logger: LoggerProtocol = RapidataLogger()  # type: ignore[assignment]



from rapidata.rapidata_client.filter._base_filter import RapidataFilter
from rapidata.api_client.models.language_user_filter_model import (
    LanguageUserFilterModel,
)
from pydantic import BaseModel, field_validator
from rapidata.rapidata_client.config import logger


class LanguageFilter(RapidataFilter, BaseModel):
    """LanguageFilter Class

    Can be used to filter who to target based on language codes.

    Args:
        language_codes (list[str]): List of language codes to filter by.

    Example:
        ```python
        LanguageFilter(["en", "de"])
        ```
        This will limit the order to be shown to only people who have their phone set to english or german
    """

    language_codes: list[str]

    def __init__(self, language_codes: list[str]):
        super().__init__(language_codes=language_codes)

    @field_validator("language_codes")
    @classmethod
    def validate_language_codes(cls, codes: list[str]) -> list[str]:
        validated = []
        for code in codes:
            if len(code) != 2:
                raise ValueError(
                    f"Language codes must be length 2. Invalid code: '{code}'"
                )
            if code != code.lower():
                logger.warning(
                    f"Language code '{code}' should be lowercase. It will be lowercased automatically."
                )
            validated.append(code.lower())
        return validated

    def _to_model(self):
        return LanguageUserFilterModel(
            _t="LanguageFilter",
            languages=self.language_codes,
        )



import re
import os
from rapidata.api_client.models.existing_asset_input import ExistingAssetInput
from rapidata.api_client.models.multi_asset_input import (
    MultiAssetInput,
    MultiAssetInputAssetsInner,
)
from rapidata.api_client.models.text_asset_input import TextAssetInput
from rapidata.service.openapi_service import OpenAPIService
from rapidata.rapidata_client.config import logger
from rapidata.rapidata_client.config import tracer
from rapidata.rapidata_client.config import rapidata_config
from cachetools import LRUCache


class AssetUploader:
    _shared_upload_cache: LRUCache = LRUCache(maxsize=100_000)

    def __init__(self, openapi_service: OpenAPIService):
        self.openapi_service = openapi_service

    def _get_cache_key(self, asset: str) -> str:
        """Generate cache key for an asset."""
        if re.match(r"^https?://", asset):
            return asset
        else:
            if not os.path.exists(asset):
                raise FileNotFoundError(f"File not found: {asset}")
            
            stat = os.stat(asset)
            # Combine path, size, and modification time
            return f"{asset}:{stat.st_size}:{stat.st_mtime_ns}"

    def upload_asset(self, asset: str) -> str:
        with tracer.start_as_current_span("AssetUploader.upload_asset"):
            logger.debug("Uploading asset: %s", asset)
            assert isinstance(asset, str), "Asset must be a string"
            
            asset_key = self._get_cache_key(asset)
            if asset_key in self._shared_upload_cache:
                logger.debug("Asset found in cache")
                return self._shared_upload_cache[asset_key]

            if re.match(r"^https?://", asset):
                response = self.openapi_service.asset_api.asset_url_post(
                    url=asset,
                )
            else:
                response = self.openapi_service.asset_api.asset_file_post(
                    file=asset,
                )
            logger.info("Asset uploaded: %s", response.file_name)
            if rapidata_config.upload.cacheUploads:
                self._shared_upload_cache[asset_key] = response.file_name
                logger.debug("Asset added to cache")
            return response.file_name

    def get_uploaded_text_input(
        self, assets: list[str] | str
    ) -> MultiAssetInput | TextAssetInput:
        if isinstance(assets, list):
            return MultiAssetInput(
                _t="MultiAssetInput",
                assets=[
                    MultiAssetInputAssetsInner(
                        actual_instance=TextAssetInput(_t="TextAssetInput", text=asset)
                    )
                    for asset in assets
                ],
            )
        else:
            return TextAssetInput(_t="TextAssetInput", text=assets)

    def get_uploaded_asset_input(
        self, assets: list[str] | str
    ) -> MultiAssetInput | ExistingAssetInput:
        if isinstance(assets, list):
            return MultiAssetInput(
                _t="MultiAssetInput",
                assets=[
                    MultiAssetInputAssetsInner(
                        actual_instance=ExistingAssetInput(
                            _t="ExistingAssetInput",
                            name=self.upload_asset(asset),
                        ),
                    )
                    for asset in assets
                ],
            )
        else:
            return ExistingAssetInput(
                _t="ExistingAssetInput",
                name=self.upload_asset(assets),
            )



from itertools import zip_longest
from typing import Literal, cast, Iterable
from rapidata.rapidata_client.datapoints._datapoint import Datapoint


class DatapointsValidator:
    @staticmethod
    def validate_datapoints(
        datapoints: list[str] | list[list[str]],
        contexts: list[str] | None = None,
        media_contexts: list[str] | None = None,
        sentences: list[str] | None = None,
        private_notes: list[str] | None = None,
        groups: list[str] | None = None,
    ) -> None:
        if contexts and len(contexts) != len(datapoints):
            raise ValueError("Number of contexts must match number of datapoints")
        if media_contexts and len(media_contexts) != len(datapoints):
            raise ValueError("Number of media contexts must match number of datapoints")
        if sentences and len(sentences) != len(datapoints):
            raise ValueError("Number of sentences must match number of datapoints")
        if private_notes and len(private_notes) != len(datapoints):
            raise ValueError("Number of private notes must match number of datapoints")
        if groups and (
            len(groups) != len(datapoints) or len(groups) != len(set(groups))
        ):
            raise ValueError(
                "Number of groups must match number of datapoints and must be unique."
            )

    @staticmethod
    def map_datapoints(
        datapoints: list[str] | list[list[str]],
        contexts: list[str] | None = None,
        media_contexts: list[str] | None = None,
        sentences: list[str] | None = None,
        private_notes: list[str] | None = None,
        groups: list[str] | None = None,
        data_type: Literal["text", "media"] = "media",
    ) -> list[Datapoint]:
        DatapointsValidator.validate_datapoints(
            datapoints=datapoints,
            contexts=contexts,
            media_contexts=media_contexts,
            sentences=sentences,
            private_notes=private_notes,
            groups=groups,
        )
        return [
            Datapoint(
                asset=asset,
                data_type=data_type,
                context=context,
                media_context=media_context,
                sentence=sentence,
                private_note=private_note,
                group=group,
            )
            for asset, context, media_context, sentence, private_note, group in cast(
                "Iterable[tuple[str | list[str], str | None, str | None, str | None, str | None, str | None]]",  # because iterator only supports 5 arguments with specific type casting
                zip_longest(
                    datapoints,
                    contexts or [],
                    media_contexts or [],
                    sentences or [],
                    private_notes or [],
                    groups or [],
                ),
            )
        ]



import urllib.parse
import webbrowser
from colorama import Fore
import pandas as pd
from typing import Literal, Optional

from rapidata.rapidata_client.config import logger, managed_print, tracer
from rapidata.rapidata_client.benchmark._detail_mapper import DetailMapper
from rapidata.service.openapi_service import OpenAPIService
from rapidata.api_client.models.update_leaderboard_model import UpdateLeaderboardModel


class RapidataLeaderboard:
    """
    An instance of a Rapidata leaderboard.

    Used to interact with a specific leaderboard in the Rapidata system, such as retrieving prompts and evaluating models.

    Args:
        name: The name that will be used to identify the leaderboard on the overview.
        instruction: The instruction that will determine what how the models will be evaluated.
        show_prompt: Whether to show the prompt to the users.
        id: The ID of the leaderboard.
        openapi_service: The OpenAPIService instance for API interaction.
    """

    def __init__(
        self,
        name: str,
        instruction: str,
        show_prompt: bool,
        show_prompt_asset: bool,
        inverse_ranking: bool,
        response_budget: int,
        min_responses_per_matchup: int,
        benchmark_id: str,
        id: str,
        openapi_service: OpenAPIService,
    ):
        self.__openapi_service = openapi_service
        self.__name = name
        self.__instruction = instruction
        self.__show_prompt = show_prompt
        self.__show_prompt_asset = show_prompt_asset
        self.__inverse_ranking = inverse_ranking
        self.__response_budget = response_budget
        self.__min_responses_per_matchup = min_responses_per_matchup
        self.__benchmark_id = benchmark_id
        self.id = id
        self.__leaderboard_page = f"https://app.{self.__openapi_service.environment}/mri/benchmarks/{self.__benchmark_id}/leaderboard/{self.id}"

    @property
    def level_of_detail(self) -> Literal["low", "medium", "high", "very high"]:
        """
        Returns the level of detail of the leaderboard.
        """
        return DetailMapper.get_level_of_detail(self.__response_budget)

    @level_of_detail.setter
    def level_of_detail(
        self, level_of_detail: Literal["low", "medium", "high", "very high"]
    ):
        """
        Sets the level of detail of the leaderboard.
        """
        with tracer.start_as_current_span("RapidataLeaderboard.level_of_detail.setter"):
            logger.debug(f"Setting level of detail to {level_of_detail}")
            self.__response_budget = DetailMapper.get_budget(level_of_detail)
            self._update_config()

    @property
    def min_responses_per_matchup(self) -> int:
        """
        Returns the minimum number of responses required to be considered for the leaderboard.
        """
        return self.__min_responses_per_matchup

    @min_responses_per_matchup.setter
    def min_responses_per_matchup(self, min_responses: int):
        """
        Sets the minimum number of responses required to be considered for the leaderboard.
        """
        with tracer.start_as_current_span(
            "RapidataLeaderboard.min_responses_per_matchup.setter"
        ):
            if not isinstance(min_responses, int):
                raise ValueError("Min responses per matchup must be an integer")

            if min_responses < 3:
                raise ValueError("Min responses per matchup must be at least 3")

            logger.debug(
                f"Setting min responses per matchup to {min_responses} for leaderboard {self.name}"
            )
            self.__min_responses_per_matchup = min_responses
            self._update_config()

    @property
    def show_prompt_asset(self) -> bool:
        """
        Returns whether the prompt asset is shown to the users.
        """
        return self.__show_prompt_asset

    @property
    def inverse_ranking(self) -> bool:
        """
        Returns whether the ranking is inverse.
        """
        return self.__inverse_ranking

    @property
    def show_prompt(self) -> bool:
        """
        Returns whether the prompt is shown to the users.
        """
        return self.__show_prompt

    @property
    def instruction(self) -> str:
        """
        Returns the instruction of the leaderboard.
        """
        return self.__instruction

    @property
    def name(self) -> str:
        """
        Returns the name of the leaderboard.
        """
        return self.__name

    @name.setter
    def name(self, name: str):
        """
        Sets the name of the leaderboard.
        """
        with tracer.start_as_current_span("RapidataLeaderboard.name.setter"):
            if not isinstance(name, str):
                raise ValueError("Name must be a string")
            if len(name) < 1:
                raise ValueError("Name must be at least 1 character long")

            self.__name = name
            self._update_config()

    def get_standings(self, tags: Optional[list[str]] = None) -> pd.DataFrame:
        """
        Returns the standings of the leaderboard.

        Args:
            tags: The matchups with these tags should be used to create the standings.
                If tags are None, all matchups will be considered.
                If tags are empty, no matchups will be considered.

        Returns:
            A pandas DataFrame containing the standings of the leaderboard.
        """
        with tracer.start_as_current_span("RapidataLeaderboard.get_standings"):
            participants = self.__openapi_service.leaderboard_api.leaderboard_leaderboard_id_standings_get(
                leaderboard_id=self.id, tags=tags
            )

            standings = []
            for participant in participants.items:
                standings.append(
                    {
                        "name": participant.name,
                        "wins": participant.wins,
                        "total_matches": participant.total_matches,
                        "score": (
                            round(participant.score, 2)
                            if participant.score is not None
                            else None
                        ),
                    }
                )

            return pd.DataFrame(standings)

    def view(self) -> None:
        """
        Views the leaderboard.
        """
        logger.info("Opening leaderboard page in browser...")
        could_open_browser = webbrowser.open(self.__leaderboard_page)
        if not could_open_browser:
            encoded_url = urllib.parse.quote(
                self.__leaderboard_page, safe="%/:=&?~#+!$,;'@()*[]"
            )
            managed_print(
                Fore.RED
                + f"Please open this URL in your browser: '{encoded_url}'"
                + Fore.RESET
            )

    def _custom_config(self, response_budget: int, min_responses_per_matchup: int):
        self.__response_budget = response_budget
        self.__min_responses_per_matchup = min_responses_per_matchup
        self._update_config()

    def _update_config(self):
        with tracer.start_as_current_span("RapidataLeaderboard._update_config"):
            self.__openapi_service.leaderboard_api.leaderboard_leaderboard_id_patch(
                leaderboard_id=self.id,
                update_leaderboard_model=UpdateLeaderboardModel(
                    name=self.__name,
                    responseBudget=self.__response_budget,
                    minResponses=self.__min_responses_per_matchup,
                ),
            )

    def __str__(self) -> str:
        return f"RapidataLeaderboard(name={self.name}, instruction={self.instruction}, show_prompt={self.show_prompt}, leaderboard_id={self.id})"

    def __repr__(self) -> str:
        return self.__str__()



from rapidata.rapidata_client.settings._rapidata_setting import RapidataSetting


class SwapContextInstruction(RapidataSetting):
    """
    Swap the place of the context and instruction.

    If set to true, the instruction will be shown on top and the context below. if collapsed, only the instruction will be shown.

    By default, the context will be shown on top and the instruction below.

    Args:
        value (bool): Whether to swap the place of the context and instruction.
    """

    def __init__(self, value: bool = True):
        if not isinstance(value, bool):
            raise ValueError("The value must be a boolean.")

        super().__init__(key="swap_question_and_prompt", value=value)



from rapidata.api_client import ExistingAssetInput
from rapidata.api_client.models.create_demographic_rapid_model_asset import (
    CreateDemographicRapidModelAsset,
)
from rapidata.service.openapi_service import OpenAPIService
from rapidata.api_client.models.classify_payload import ClassifyPayload
from rapidata.api_client.models.classify_payload_category import ClassifyPayloadCategory
from rapidata.rapidata_client.config import logger
from rapidata.api_client.models.create_demographic_rapid_model import (
    CreateDemographicRapidModel,
)
from rapidata.rapidata_client.datapoints._asset_uploader import AssetUploader


class DemographicManager:
    def __init__(self, openapi_service: OpenAPIService):
        self._openapi_service = openapi_service
        self._asset_uploader = AssetUploader(openapi_service)
        logger.debug("DemographicManager initialized")

    def create_demographic_rapid(
        self, instruction: str, answer_options: list[str], datapoint: str, key: str
    ):

        model = CreateDemographicRapidModel(
            key=key,
            payload=ClassifyPayload(
                _t="ClassifyPayload",
                categories=[
                    ClassifyPayloadCategory(label=option, value=option)
                    for option in answer_options
                ],
                title=instruction,
            ),
            asset=CreateDemographicRapidModelAsset(
                actual_instance=ExistingAssetInput(
                    _t="ExistingAssetInput",
                    name=self._asset_uploader.upload_asset(datapoint),
                ),
            ),
        )

        result = self._openapi_service.rapid_api.rapid_demographic_post(
            create_demographic_rapid_model=model
        )
        logger.info(f"Demographic Rapid created: {result.rapid_id}")
        return result.rapid_id

    def __str__(self) -> str:
        return "DemographicManager"

    def __repr__(self) -> str:
        return self.__str__()



# coding: utf-8

"""
    Rapidata.Dataset

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictStr
from typing import Optional
from typing_extensions import Annotated
from rapidata.api_client.models.boost_query_result import BoostQueryResult
from rapidata.api_client.models.change_boost_model import ChangeBoostModel
from rapidata.api_client.models.paged_result_campaign_query_result import PagedResultCampaignQueryResult
from rapidata.api_client.models.query_model import QueryModel
from rapidata.api_client.models.update_campaign_model import UpdateCampaignModel

from rapidata.api_client.api_client import ApiClient, RequestSerialized
from rapidata.api_client.api_response import ApiResponse
from rapidata.api_client.rest import RESTResponseType


class CampaignApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def campaign_boost_put(
        self,
        change_boost_model: Optional[ChangeBoostModel] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Updates the boost with manual boosts.


        :param change_boost_model:
        :type change_boost_model: ChangeBoostModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_boost_put_serialize(
            change_boost_model=change_boost_model,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def campaign_boost_put_with_http_info(
        self,
        change_boost_model: Optional[ChangeBoostModel] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Updates the boost with manual boosts.


        :param change_boost_model:
        :type change_boost_model: ChangeBoostModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_boost_put_serialize(
            change_boost_model=change_boost_model,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def campaign_boost_put_without_preload_content(
        self,
        change_boost_model: Optional[ChangeBoostModel] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Updates the boost with manual boosts.


        :param change_boost_model:
        :type change_boost_model: ChangeBoostModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_boost_put_serialize(
            change_boost_model=change_boost_model,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _campaign_boost_put_serialize(
        self,
        change_boost_model,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if change_boost_model is not None:
            _body_params = change_boost_model



        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json', 
                        'text/json', 
                        'application/*+json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'bearer', 
            'oauth2'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/campaign/boost',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def campaign_boost_status_get(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> BoostQueryResult:
        """Gets the status of the boost.


        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_boost_status_get_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "BoostQueryResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def campaign_boost_status_get_with_http_info(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[BoostQueryResult]:
        """Gets the status of the boost.


        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_boost_status_get_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "BoostQueryResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def campaign_boost_status_get_without_preload_content(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Gets the status of the boost.


        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_boost_status_get_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "BoostQueryResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _campaign_boost_status_get_serialize(
        self,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/plain', 
                    'application/json', 
                    'text/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'bearer', 
            'oauth2'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/campaign/boost/status',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def campaign_campaign_id_patch(
        self,
        campaign_id: Annotated[StrictStr, Field(description="id of the campaign to update")],
        update_campaign_model: Annotated[Optional[UpdateCampaignModel], Field(description="model containing the updates")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Updates a campaign.


        :param campaign_id: id of the campaign to update (required)
        :type campaign_id: str
        :param update_campaign_model: model containing the updates
        :type update_campaign_model: UpdateCampaignModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_campaign_id_patch_serialize(
            campaign_id=campaign_id,
            update_campaign_model=update_campaign_model,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def campaign_campaign_id_patch_with_http_info(
        self,
        campaign_id: Annotated[StrictStr, Field(description="id of the campaign to update")],
        update_campaign_model: Annotated[Optional[UpdateCampaignModel], Field(description="model containing the updates")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Updates a campaign.


        :param campaign_id: id of the campaign to update (required)
        :type campaign_id: str
        :param update_campaign_model: model containing the updates
        :type update_campaign_model: UpdateCampaignModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_campaign_id_patch_serialize(
            campaign_id=campaign_id,
            update_campaign_model=update_campaign_model,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def campaign_campaign_id_patch_without_preload_content(
        self,
        campaign_id: Annotated[StrictStr, Field(description="id of the campaign to update")],
        update_campaign_model: Annotated[Optional[UpdateCampaignModel], Field(description="model containing the updates")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Updates a campaign.


        :param campaign_id: id of the campaign to update (required)
        :type campaign_id: str
        :param update_campaign_model: model containing the updates
        :type update_campaign_model: UpdateCampaignModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_campaign_id_patch_serialize(
            campaign_id=campaign_id,
            update_campaign_model=update_campaign_model,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _campaign_campaign_id_patch_serialize(
        self,
        campaign_id,
        update_campaign_model,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if campaign_id is not None:
            _path_params['campaignId'] = campaign_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if update_campaign_model is not None:
            _body_params = update_campaign_model



        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json', 
                        'text/json', 
                        'application/*+json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'bearer', 
            'oauth2'
        ]

        return self.api_client.param_serialize(
            method='PATCH',
            resource_path='/campaign/{campaignId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def campaign_campaign_id_pause_post(
        self,
        campaign_id: Annotated[StrictStr, Field(description="id of the campaign that should be paused")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Pauses a campaign.


        :param campaign_id: id of the campaign that should be paused (required)
        :type campaign_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_campaign_id_pause_post_serialize(
            campaign_id=campaign_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def campaign_campaign_id_pause_post_with_http_info(
        self,
        campaign_id: Annotated[StrictStr, Field(description="id of the campaign that should be paused")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Pauses a campaign.


        :param campaign_id: id of the campaign that should be paused (required)
        :type campaign_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_campaign_id_pause_post_serialize(
            campaign_id=campaign_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def campaign_campaign_id_pause_post_without_preload_content(
        self,
        campaign_id: Annotated[StrictStr, Field(description="id of the campaign that should be paused")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Pauses a campaign.


        :param campaign_id: id of the campaign that should be paused (required)
        :type campaign_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_campaign_id_pause_post_serialize(
            campaign_id=campaign_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _campaign_campaign_id_pause_post_serialize(
        self,
        campaign_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if campaign_id is not None:
            _path_params['campaignId'] = campaign_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter




        # authentication setting
        _auth_settings: List[str] = [
            'bearer', 
            'oauth2'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/campaign/{campaignId}/pause',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def campaign_campaign_id_resume_post(
        self,
        campaign_id: Annotated[StrictStr, Field(description="id of the campaign that should be paused")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Resumes a campaign.


        :param campaign_id: id of the campaign that should be paused (required)
        :type campaign_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_campaign_id_resume_post_serialize(
            campaign_id=campaign_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def campaign_campaign_id_resume_post_with_http_info(
        self,
        campaign_id: Annotated[StrictStr, Field(description="id of the campaign that should be paused")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Resumes a campaign.


        :param campaign_id: id of the campaign that should be paused (required)
        :type campaign_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_campaign_id_resume_post_serialize(
            campaign_id=campaign_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def campaign_campaign_id_resume_post_without_preload_content(
        self,
        campaign_id: Annotated[StrictStr, Field(description="id of the campaign that should be paused")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Resumes a campaign.


        :param campaign_id: id of the campaign that should be paused (required)
        :type campaign_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_campaign_id_resume_post_serialize(
            campaign_id=campaign_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _campaign_campaign_id_resume_post_serialize(
        self,
        campaign_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if campaign_id is not None:
            _path_params['campaignId'] = campaign_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter




        # authentication setting
        _auth_settings: List[str] = [
            'bearer', 
            'oauth2'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/campaign/{campaignId}/resume',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def campaign_monitor_get(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """The monitor endpoint is used to monitor the health of the service


        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_monitor_get_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def campaign_monitor_get_with_http_info(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """The monitor endpoint is used to monitor the health of the service


        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_monitor_get_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def campaign_monitor_get_without_preload_content(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """The monitor endpoint is used to monitor the health of the service


        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaign_monitor_get_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _campaign_monitor_get_serialize(
        self,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter




        # authentication setting
        _auth_settings: List[str] = [
            'bearer', 
            'oauth2'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/campaign/monitor',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def campaigns_get(
        self,
        request: Annotated[Optional[QueryModel], Field(description="The parameters for filtering, paging, and sorting")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PagedResultCampaignQueryResult:
        """Queries orders based on a filter, page, and sort criteria.


        :param request: The parameters for filtering, paging, and sorting
        :type request: QueryModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaigns_get_serialize(
            request=request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PagedResultCampaignQueryResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def campaigns_get_with_http_info(
        self,
        request: Annotated[Optional[QueryModel], Field(description="The parameters for filtering, paging, and sorting")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PagedResultCampaignQueryResult]:
        """Queries orders based on a filter, page, and sort criteria.


        :param request: The parameters for filtering, paging, and sorting
        :type request: QueryModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaigns_get_serialize(
            request=request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PagedResultCampaignQueryResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def campaigns_get_without_preload_content(
        self,
        request: Annotated[Optional[QueryModel], Field(description="The parameters for filtering, paging, and sorting")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Queries orders based on a filter, page, and sort criteria.


        :param request: The parameters for filtering, paging, and sorting
        :type request: QueryModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._campaigns_get_serialize(
            request=request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PagedResultCampaignQueryResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _campaigns_get_serialize(
        self,
        request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if request is not None:
            
            _query_params.append(('request', request))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/plain', 
                    'application/json', 
                    'text/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'bearer', 
            'oauth2'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/campaigns',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





from typing import Any
from rapidata.rapidata_client.filter._base_filter import RapidataFilter
from rapidata.api_client.models.not_user_filter_model import NotUserFilterModel
from rapidata.api_client.models.and_user_filter_model_filters_inner import (
    AndUserFilterModelFiltersInner,
)
from pydantic import BaseModel, ConfigDict


class NotFilter(RapidataFilter, BaseModel):
    """A filter that negates another filter's condition.
    This class implements a logical NOT operation on a given filter, inverting its results.

    Args:
        filter (RapidataFilter): The filter whose condition should be negated.

    Example:
        ```python
        from rapidata import NotFilter, LanguageFilter

        NotFilter(LanguageFilter(["en"]))
        ```

        This will limit the order to be shown to only people who have their phone set to a language other than English.
    """

    model_config = ConfigDict(arbitrary_types_allowed=True)

    filter: RapidataFilter

    def __init__(self, filter: RapidataFilter):
        super().__init__(filter=filter)

    def _to_model(self):
        return NotUserFilterModel(
            _t="NotFilter",
            filter=AndUserFilterModelFiltersInner(self.filter._to_model()),
        )



# Standard library imports
import json
import urllib.parse
import webbrowser
from time import sleep
from typing import cast
from colorama import Fore
from datetime import datetime
from tqdm import tqdm

# Local/application imports
from rapidata.api_client.exceptions import ApiException
from rapidata.api_client.models.campaign_artifact_model import CampaignArtifactModel
from rapidata.api_client.models.order_state import OrderState
from rapidata.api_client.models.preview_order_model import PreviewOrderModel
from rapidata.api_client.models.submit_order_model import SubmitOrderModel
from rapidata.api_client.models.preliminary_download_model import (
    PreliminaryDownloadModel,
)
from rapidata.api_client.models.workflow_artifact_model import WorkflowArtifactModel
from rapidata.rapidata_client.order.rapidata_results import RapidataResults
from rapidata.service.openapi_service import OpenAPIService
from rapidata.rapidata_client.config import (
    logger,
    managed_print,
    rapidata_config,
    tracer,
)
from rapidata.rapidata_client.api.rapidata_api_client import (
    suppress_rapidata_error_logging,
)


class RapidataOrder:
    """
    An instance of a Rapidata order.

    Used to interact with a specific order in the Rapidata system, such as starting,
    pausing, and retrieving results.

    Args:
        name: The name of the order.
        order_id: The ID of the order.
        openapi_service: The OpenAPIService instance for API interaction.
    """

    def __init__(
        self,
        name: str,
        order_id: str,
        openapi_service: OpenAPIService,
    ):
        self.id = order_id
        self.name = name
        self.__created_at: datetime | None = None
        self._openapi_service = openapi_service
        self.__workflow_id: str = ""
        self.__campaign_id: str = ""
        self.__pipeline_id: str = ""
        self._max_retries = 10
        self._retry_delay = 2
        self.order_details_page = (
            f"https://app.{self._openapi_service.environment}/order/detail/{self.id}"
        )
        logger.debug("RapidataOrder initialized")

    @property
    def created_at(self) -> datetime:
        """Returns the creation date of the order."""
        if not self.__created_at:
            self.__created_at = self._openapi_service.order_api.order_order_id_get(
                self.id
            ).order_date
        return self.__created_at

    def run(self) -> "RapidataOrder":
        """Runs the order to start collecting responses."""
        with tracer.start_as_current_span("RapidataOrder.run"):
            logger.info("Starting order '%s'", self)
            self._openapi_service.order_api.order_order_id_submit_post(
                self.id, SubmitOrderModel(ignoreFailedDatapoints=True)
            )
            logger.debug("Order '%s' has been started.", self)
            managed_print(
                f"Order '{self.name}' is now viewable under: {self.order_details_page}"
            )
            return self

    def pause(self) -> None:
        """Pauses the order."""
        with tracer.start_as_current_span("RapidataOrder.pause"):
            logger.info("Pausing order '%s'", self)
            self._openapi_service.order_api.order_order_id_pause_post(self.id)
            logger.debug("Order '%s' has been paused.", self)
            managed_print(f"Order '{self}' has been paused.")

    def unpause(self) -> None:
        """Unpauses/resumes the order."""
        with tracer.start_as_current_span("RapidataOrder.unpause"):
            logger.info("Unpausing order '%s'", self)
            self._openapi_service.order_api.order_order_id_resume_post(self.id)
            logger.debug("Order '%s' has been unpaused.", self)
            managed_print(f"Order '{self}' has been unpaused.")

    def delete(self) -> None:
        """Deletes the order."""
        with tracer.start_as_current_span("RapidataOrder.delete"):
            logger.info("Deleting order '%s'", self)
            self._openapi_service.order_api.order_order_id_delete(self.id)
            logger.debug("Order '%s' has been deleted.", self)
            managed_print(f"Order '{self}' has been deleted.")

    def get_status(self) -> str:
        """
        Gets the status of the order.

        States:
            Created: The order has been created but not started yet.\n
            Preview: The order has been set up and ready but not collecting responses yet.\n
            Submitted: The order has been submitted and is being reviewed.\n
            ManualReview: The order is in manual review - something went wrong with the automatic approval.\n
            Processing: The order is actively being processed.\n
            Paused: The order has been paused.\n
            Completed: The order has been completed.\n
            Failed: The order has failed.
        """
        with tracer.start_as_current_span("RapidataOrder.get_status"):
            return self._openapi_service.order_api.order_order_id_get(self.id).state

    def display_progress_bar(self, refresh_rate: int = 5) -> None:
        """
        Displays a progress bar for the order processing using tqdm.

        Args:
            refresh_rate: How often to refresh the progress bar, in seconds.
        """
        if refresh_rate < 1:
            raise ValueError("refresh_rate must be at least 1")

        if self.get_status() == OrderState.CREATED:
            raise Exception("Order has not been started yet. Please start it first.")

        while self.get_status() == OrderState.SUBMITTED:
            managed_print(
                f"Order '{self}' is submitted and being reviewed. Standby...", end="\r"
            )
            sleep(1)

        if self.get_status() == OrderState.MANUALREVIEW:
            raise Exception(
                f"Order '{self}' is in manual review. It might take some time to start. To speed up the process, contact support (info@rapidata.ai).\nOnce started, run this method again to display the progress bar."
            )

        with tqdm(
            total=100,
            desc="Processing order",
            unit="%",
            bar_format="{desc}: {percentage:3.0f}%|{bar}| completed [{elapsed}<{remaining}, {rate_fmt}]",
            disable=rapidata_config.logging.silent_mode,
        ) as pbar:
            last_percentage = 0
            while True:
                current_percentage = self._workflow_progress.completion_percentage
                if current_percentage > last_percentage:
                    pbar.update(current_percentage - last_percentage)
                    last_percentage = current_percentage

                if current_percentage >= 100:
                    break

                sleep(refresh_rate)

    @property
    def _workflow_progress(self):
        """
        Gets the workflow progress.
        """
        progress = None
        for _ in range(self._max_retries // 2):
            try:
                with suppress_rapidata_error_logging():
                    workflow_id = self.__get_workflow_id()
                    progress = self._openapi_service.workflow_api.workflow_workflow_id_progress_get(
                        workflow_id
                    )
                break
            except Exception:
                sleep(self._retry_delay * 2)
        if not progress:
            raise Exception("Failed to get progress. Please try again later.")
        return progress

    def get_results(self, preliminary_results: bool = False) -> RapidataResults:
        """
        Gets the results of the order.
        If the order is still processing, this method will block until the order is completed and then return the results.

        Args:
            preliminary_results: If True, returns the preliminary results of the order. Defaults to False.
                Note that preliminary results are not final and may not contain all the datapoints & responses. Only the onese that are already available.
        """
        with tracer.start_as_current_span("RapidataOrder.get_results"):
            logger.info("Getting results for order '%s'...", self)
            if preliminary_results and self.get_status() not in [OrderState.COMPLETED]:
                return self.__get_preliminary_results()

            elif preliminary_results and self.get_status() in [OrderState.COMPLETED]:
                managed_print("Order is already completed. Returning final results.")

            while (state := self.get_status()) not in [
                OrderState.COMPLETED,
                OrderState.PAUSED,
                OrderState.MANUALREVIEW,
                OrderState.FAILED,
            ]:
                sleep(5)
                logger.debug(
                    "Order '%s' is in state %s not yet completed. Waiting...",
                    self,
                    state,
                )

            try:
                return RapidataResults(
                    json.loads(
                        self._openapi_service.order_api.order_order_id_download_results_get(
                            order_id=self.id
                        )
                    )
                )
            except (ApiException, json.JSONDecodeError) as e:
                raise Exception(f"Failed to get order results: {str(e)}") from e

    def view(self) -> None:
        """
        Opens the order details page in the browser.

        Raises:
            Exception: If the order is not in processing state.
        """
        logger.info("Opening order details page in browser...")
        could_open_browser = webbrowser.open(self.order_details_page)
        if not could_open_browser:
            encoded_url = urllib.parse.quote(
                self.order_details_page, safe="%/:=&?~#+!$,;'@()*[]"
            )
            managed_print(
                Fore.RED
                + f"Please open this URL in your browser: '{encoded_url}'"
                + Fore.RESET
            )

    def preview(self) -> None:
        """
        Opens a preview of the order in the browser.

        Raises:
            Exception: If the order is not in processing state.
        """
        logger.info("Opening order preview in browser...")
        if self.get_status() == OrderState.CREATED:
            logger.info("Order is still in state created. Setting it to preview.")
            self._openapi_service.order_api.order_order_id_preview_post(
                self.id, PreviewOrderModel(ignoreFailedDatapoints=True)
            )
            logger.info("Order is now in preview state.")

        campaign_id = self.__get_campaign_id()
        auth_url = f"https://app.{self._openapi_service.environment}/order/detail/{self.id}/preview?campaignId={campaign_id}"
        could_open_browser = webbrowser.open(auth_url)
        if not could_open_browser:
            encoded_url = urllib.parse.quote(auth_url, safe="%/:=&?~#+!$,;'@()*[]")
            managed_print(
                Fore.RED
                + f"Please open this URL in your browser: '{encoded_url}'"
                + Fore.RESET
            )

    def __get_pipeline_id(self) -> str:
        """Internal method to fetch and cache the pipeline ID."""
        if not self.__pipeline_id:
            for _ in range(self._max_retries):
                try:
                    self.__pipeline_id = (
                        self._openapi_service.order_api.order_order_id_get(
                            self.id
                        ).pipeline_id
                    )
                    break
                except Exception:
                    sleep(self._retry_delay)
            else:
                raise Exception("Failed to fetch pipeline ID after retries.")
        return self.__pipeline_id

    def __get_workflow_id(self) -> str:
        """Internal method to fetch and cache the workflow ID."""
        if not self.__workflow_id:
            self.__fetch_workflow_and_campaign_ids()
        return self.__workflow_id

    def __get_campaign_id(self) -> str:
        """Internal method to fetch and cache the campaign ID."""
        if not self.__campaign_id:
            self.__fetch_workflow_and_campaign_ids()
        return self.__campaign_id

    def __fetch_workflow_and_campaign_ids(self) -> None:
        """Fetches workflow and campaign IDs from the pipeline."""
        if self.__workflow_id and self.__campaign_id:
            return
        pipeline_id = self.__get_pipeline_id()
        for _ in range(self._max_retries):
            try:
                pipeline = self._openapi_service.pipeline_api.pipeline_pipeline_id_get(
                    pipeline_id
                )
                self.__workflow_id = cast(
                    WorkflowArtifactModel,
                    pipeline.artifacts["workflow-artifact"].actual_instance,
                ).workflow_id
                self.__campaign_id = cast(
                    CampaignArtifactModel,
                    pipeline.artifacts["campaign-artifact"].actual_instance,
                ).campaign_id
                return
            except Exception:
                sleep(self._retry_delay)
        raise Exception("Failed to fetch workflow and campaign IDs after retries.")

    def __get_preliminary_results(self) -> RapidataResults:
        """Internal method to fetch preliminary results."""
        try:
            pipeline_id = self.__get_pipeline_id()
            download_id = self._openapi_service.pipeline_api.pipeline_pipeline_id_preliminary_download_post(
                pipeline_id, PreliminaryDownloadModel(sendEmail=False)
            ).download_id

            elapsed = 0
            timeout = 60
            while elapsed < timeout:
                preliminary_results = self._openapi_service.pipeline_api.pipeline_preliminary_download_preliminary_download_id_get(
                    preliminary_download_id=download_id
                )
                if preliminary_results:
                    return RapidataResults(json.loads(preliminary_results.decode()))
                sleep(1)
                elapsed += 1
            raise Exception("Timeout waiting for preliminary results.")
        except (ApiException, json.JSONDecodeError) as e:
            raise Exception(f"Failed to get preliminary results: {str(e)}") from e

    def __str__(self) -> str:
        return f"RapidataOrder(name='{self.name}', order_id='{self.id}')"

    def __repr__(self) -> str:
        return f"RapidataOrder(name='{self.name}', order_id='{self.id}')"



from ._datapoint import Datapoint
from .metadata import (
    Metadata,
    PromptMetadata,
    PrivateTextMetadata,
    PublicTextMetadata,
    SelectWordsMetadata,
    MediaAssetMetadata,
    PromptIdentifierMetadata,
)
